@@ -9,6 +9,45 @@ from CRADLE.CallPeak import vari
 from CRADLE.CallPeak import calculateRC
 
 
+def setResultValues(mergedResult, pvalues, qvalues, ctrlBW, normCtrlBW, expBW, normExpBW):
+	regionChromo = mergedResult[0]
+	regionStart = int(mergedResult[1])
+	regionEnd = int(mergedResult[2])
+
+	mergedResult[4] = takeMinusLog(pvalues)
+	mergedResult[5] = takeMinusLog(qvalues)
+
+	ctrlRC, expRC = getRCFromBWs(ctrlBW, expBW, regionChromo, regionStart, regionEnd)
+
+	ctrlRCPosMean = np.nanmean(ctrlRC)
+	expRCPosMean = np.nanmean(expRC)
+	diffPos = int(expRCPosMean - ctrlRCPosMean)
+	mergedResult[6] = diffPos
+
+	cohens_D = calculateCohenD(ctrlRC, expRC)
+	if cohens_D == np.nan:
+		print(f"""
+		Warning: Pooled Std Dev of Cohen's D is 0.
+		  This could mean that all your read counts are the same or that you
+		  aren't using enough BigWig files (You need at least 3).
+
+		  Location {regionChromo}:{regionStart}-{regionEnd}
+		""")
+
+	mergedResult.extend([ctrlRCPosMean, expRCPosMean, cohens_D])
+
+	if vari.I_LOG2FC:
+		normCtrlRC, normExpRC = getRCFromBWs(normCtrlBW, normExpBW, regionChromo, regionStart, regionEnd)
+		normCtrlRCPosMean = np.nanmean(normCtrlRC)
+		normExpRCPosMean = np.nanmean(normExpRC)
+
+		peusdoLog2FC = calculatePeusdoLog2FC(ctrlRCPosMean, expRCPosMean, normCtrlRCPosMean, normExpRCPosMean)
+	else:
+		peusdoLog2FC = np.nan
+
+	mergedResult.append(peusdoLog2FC)
+
+
 def mergePeaks(peakResult):
 	## open bigwig files to calculate effect size
 	ctrlBW = [0] * vari.CTRLBW_NUM
@@ -19,6 +58,10 @@ def mergePeaks(peakResult):
 	for i in range(vari.EXPBW_NUM):
 		expBW[i] = pyBigWig.open(vari.EXPBW_NAMES[i])
 
+	if vari.I_LOG2FC:
+		normCtrlBW = [pyBigWig.open(bwName) for bwName in vari.NORM_CTRLBW_NAMES]
+		normExpBW = [pyBigWig.open(bwName) for bwName in vari.NORM_EXPBW_NAMES]
+
 	mergedPeak = []
 
 	pastChromo = peakResult[0][0]
@@ -30,46 +73,21 @@ def mergePeaks(peakResult):
 	mergedPeak.append(peakResult[0])
 	resultIdx = 0
 	if len(peakResult) == 1:
-		minPValue = np.min(pvalues)
-		if minPValue == 0:
-			mergedPeak[resultIdx][4] = np.nan
-		else:
-			mergedPeak[resultIdx][4] = np.round((-1) * np.log10(minPValue), 2)
-
-		minQValue = np.min(qvalues)
-		if minQValue == 0:
-			mergedPeak[resultIdx][5] = np.nan
-		else:
-			mergedPeak[resultIdx][5] = np.round((-1) * np.log10(minQValue), 2)
-
-		regionChromo = mergedPeak[resultIdx][0]
-		regionStart = int(mergedPeak[resultIdx][1])
-		regionEnd = int(mergedPeak[resultIdx][2])
-
-		ctrlRC = []
-		for rep in range(vari.CTRLBW_NUM):
-			rc = np.nanmean(np.array(ctrlBW[rep].values(regionChromo, regionStart, regionEnd)))
-			ctrlRC.extend([rc])
-		ctrlRCPosMean = np.nanmean(ctrlRC)
-
-		expRC = []
-		for rep in range(vari.EXPBW_NUM):
-			rc = np.nanmean(np.array(expBW[rep].values(regionChromo, regionStart, regionEnd)))
-			expRC.extend([rc])
-		expRCPosMean = np.nanmean(expRC)
-
-		diffPos = int(expRCPosMean - ctrlRCPosMean)
-		cohens_D = calculateCohenD(ctrlRC, expRC)
-		mergedPeak[resultIdx][6] = diffPos
-		mergedPeak[resultIdx].extend([ctrlRCPosMean, expRCPosMean, cohens_D])
+		setResultValues(mergedPeak[resultIdx], pvalues, qvalues, ctrlBW, normCtrlBW, expBW, normExpBW)
 
 		for i in range(vari.CTRLBW_NUM):
 			ctrlBW[i].close()
+
+			if vari.I_LOG2FC:
+				normCtrlBW[i].close()
+
 		for i in range(vari.EXPBW_NUM):
 			expBW[i].close()
 
-		return mergedPeak
+			if vari.I_LOG2FC:
+				normExpBW[i].close()
 
+		return mergedPeak
 
 	i = 1
 	while i < len(peakResult):
@@ -82,43 +100,10 @@ def mergePeaks(peakResult):
 
 		if (currChromo == pastChromo) and (currEnrich == pastEnrich) and ( (currStart-pastEnd) <= vari.DISTANCE):
 			mergedPeak[resultIdx][2] = currEnd
-			pvalues.extend([ currpvalue ])
-			qvalues.extend([ currqvalue ])
+			pvalues.append(currpvalue)
+			qvalues.append(currqvalue)
 		else:
-			## update the continuous regions
-			minPValue = np.min(pvalues)
-			if(minPValue == 0):
-				mergedPeak[resultIdx][4] = np.nan
-			else:
-				mergedPeak[resultIdx][4] = np.round((-1) * np.log10(minPValue), 2)
-
-			minQValue = np.min(qvalues)
-			if(minQValue == 0):
-				mergedPeak[resultIdx][5] = np.nan
-			else:
-				mergedPeak[resultIdx][5] = np.round((-1) * np.log10(minQValue), 2)
-
-			regionChromo = mergedPeak[resultIdx][0]
-			regionStart = int(mergedPeak[resultIdx][1])
-			regionEnd = int(mergedPeak[resultIdx][2])
-
-			ctrlRC = []
-			for rep in range(vari.CTRLBW_NUM):
-				rc = np.nanmean(np.array(ctrlBW[rep].values(regionChromo, regionStart, regionEnd)))
-				ctrlRC.extend([rc])
-			ctrlRCPosMean = np.nanmean(ctrlRC)
-
-			expRC = []
-			for rep in range(vari.EXPBW_NUM):
-				rc = np.nanmean(np.array(expBW[rep].values(regionChromo, regionStart, regionEnd)))
-				expRC.extend([rc])
-			expRCPosMean = np.nanmean(expRC)
-
-			diffPos = int(expRCPosMean - ctrlRCPosMean)
-			cohens_D = calculateCohenD(ctrlRC, expRC)
-			mergedPeak[resultIdx][6] = diffPos
-			mergedPeak[resultIdx].extend([ctrlRCPosMean, expRCPosMean, cohens_D])
-
+			setResultValues(mergedPeak[resultIdx], pvalues, qvalues, ctrlBW, normCtrlBW, expBW, normExpBW)
 
 			## start a new region
 			mergedPeak.append(peakResult[i])
@@ -126,39 +111,8 @@ def mergePeaks(peakResult):
 			qvalues = [currqvalue]
 			resultIdx = resultIdx + 1
 
-		if i == (len(peakResult) -1):
-			minPValue = np.min(pvalues)
-			if(minPValue == 0):
-				mergedPeak[resultIdx][4] = np.nan
-			else:
-				mergedPeak[resultIdx][4] = np.round((-1) * np.log10(minPValue), 2)
-
-			minQValue = np.min(qvalues)
-			if(minQValue == 0):
-				mergedPeak[resultIdx][5] = np.nan
-			else:
-				mergedPeak[resultIdx][5] = np.round((-1) * np.log10(minQValue), 2)
-
-			regionChromo = mergedPeak[resultIdx][0]
-			regionStart = int(mergedPeak[resultIdx][1])
-			regionEnd = int(mergedPeak[resultIdx][2])
-
-			ctrlRC = []
-			for rep in range(vari.CTRLBW_NUM):
-				rc = np.nanmean(np.array(ctrlBW[rep].values(regionChromo, regionStart, regionEnd)))
-				ctrlRC.extend([rc])
-			ctrlRCPosMean = np.nanmean(ctrlRC)
-
-			expRC = []
-			for rep in range(vari.EXPBW_NUM):
-				rc = np.nanmean(np.array(expBW[rep].values(regionChromo, regionStart, regionEnd)))
-				expRC.extend([rc])
-			expRCPosMean = np.nanmean(expRC)
-
-			diffPos = int(expRCPosMean - ctrlRCPosMean)
-			mergedPeak[resultIdx][6] = diffPos
-			cohens_D = calculateCohenD(ctrlRC, expRC)
-			mergedPeak[resultIdx].extend([ctrlRCPosMean, expRCPosMean, cohens_D])
+		if i == (len(peakResult) - 1):
+			setResultValues(mergedPeak[resultIdx], pvalues, qvalues, ctrlBW, normCtrlBW, expBW, normExpBW)
 
 		pastChromo = currChromo
 		pastEnd = currEnd
@@ -168,11 +122,28 @@ def mergePeaks(peakResult):
 
 	for i in range(vari.CTRLBW_NUM):
 		ctrlBW[i].close()
+
+		if vari.I_LOG2FC:
+			normCtrlBW[i].close()
+
 	for i in range(vari.EXPBW_NUM):
 		expBW[i].close()
 
+		if vari.I_LOG2FC:
+			normExpBW[i].close()
+
 	return mergedPeak
 
+def takeMinusLog(values):
+	minValue = np.min(values)
+
+	return np.nan if minValue == 0 else np.round((-1) * np.log10(minValue), 2)
+
+def getRCFromBWs(ctrlBW, expBW, regionChromo, regionStart, regionEnd):
+	ctrlRC = [np.nanmean(np.array(bw.values(regionChromo, regionStart, regionEnd))) for bw in ctrlBW]
+	expRC = [np.nanmean(np.array(bw.values(regionChromo, regionStart, regionEnd))) for bw in expBW]
+
+	return ctrlRC, expRC
 
 def calculateCohenD(ctrlRC, expRC):
 	dof = vari.CTRLBW_NUM + vari.EXPBW_NUM - 2
@@ -182,14 +153,21 @@ def calculateCohenD(ctrlRC, expRC):
 
 	s = np.sqrt( (  (vari.CTRLBW_NUM-1)*np.power(np.std(ctrlRC, ddof=1), 2) + (vari.EXPBW_NUM-1)*np.power(np.std(expRC, ddof=1), 2)  ) / dof )
 
+	if s == 0:
+		return np.nan
+
 	cohenD = (expRC_mean - ctrlRC_mean) / s
 
 	return cohenD
 
+def calculatePeusdoLog2FC(ctrlRCPosMean, expRCPosMean, normCtrlRCPosMean, normExpRCPosMean):
+	constant = np.max([normExpRCPosMean - expRCPosMean, normCtrlRCPosMean - ctrlRCPosMean])
+	fc = (expRCPosMean + constant) / (ctrlRCPosMean + constant)
+	peusdoLog2FC = np.log2(fc)
+
+	return peusdoLog2FC
 
 def filterSmallPeaks(peakResult):
-
-
 	maxNegLogPValue = 1
 	maxNegLogQValue = 1
 
@@ -204,12 +182,12 @@ def filterSmallPeaks(peakResult):
 			neglogPvalue = float(peakResult[i][4])
 			neglogQvalue = float(peakResult[i][5])
 
-			if(np.isnan(neglogPvalue) == False):
-				if(neglogPvalue > maxNegLogPValue):
+			if not np.isnan(neglogPvalue):
+				if neglogPvalue > maxNegLogPValue:
 					maxNegLogPValue = neglogPvalue
 
-			if(np.isnan(neglogQvalue) == False):
-				if(neglogQvalue > maxNegLogQValue):
+			if not np.isnan(neglogQvalue):
+				if neglogQvalue > maxNegLogQValue:
 					maxNegLogQValue = neglogQvalue
 
 	return finalResult, maxNegLogPValue, maxNegLogQValue
@@ -368,7 +346,7 @@ def run(args):
 				continue
 
 			if regionTheta >= vari.THETA:
-				PValueSimes.extend([ regionPvalue ])
+				PValueSimes.append(regionPvalue)
 
 	PValueGroupBh = statsmodels.sandbox.stats.multicomp.multipletests(PValueSimes, alpha=vari.FDR, method='fdr_bh')[0]
 
@@ -399,7 +377,7 @@ def run(args):
 			if np.isnan(regionPvalue):
 				continue
 			if PValueGroupBh[groupPvalueIdx + selectedIdx]:
-				selectRegionIdx.extend([ regionIdx ])
+				selectRegionIdx.append(regionIdx)
 			selectedIdx = selectedIdx + 1
 
 		groupPvalueIdx = groupPvalueIdx + selectedIdx
@@ -447,7 +425,7 @@ def run(args):
 
 
 	######## WRITE A RESULT FILE
-	colNames = ["chr", "start", "end", "name", "score", "strand", "effectSize", "inputCount", "outputCount", "-log(pvalue)", "-log(qvalue)", "cohen's_d" ]
+	colNames = ["chr", "start", "end", "name", "score", "strand", "effectSize", "inputCount", "outputCount", "-log(pvalue)", "-log(qvalue)", "cohen's_d", "peusdoLog2FC"]
 	mergedPeaks = mergePeaks(peakResult)
 	finalResult, maxNegLogPValue, maxNegLogQValue = filterSmallPeaks(mergedPeaks)
 
@@ -476,19 +454,20 @@ def run(args):
 		outputCount = int(finalResult[i][8])
 		neglogPvalue = float(finalResult[i][4])
 		cohens_D = float(finalResult[i][9])
-		if(np.isnan(neglogPvalue) == True):
-			if(maxNegLogPValue == 1):
+		peusdoLog2FC = float(finalResult[i][10])
+		if np.isnan(neglogPvalue):
+			if maxNegLogPValue == 1:
 				neglogPvalue = "-log(0)"
 			else:
 				neglogPvalue = maxNegLogPValue
 		neglogQvalue = float(finalResult[i][5])
-		if(np.isnan(neglogQvalue) == True):
-			if(maxNegLogQValue == 1):
+		if np.isnan(neglogQvalue):
+			if maxNegLogQValue == 1:
 				neglogQvalue = "-log(0)"
 			else:
 				neglogQvalue = maxNegLogQValue
 
-		peakToAdd = [chromo, start, end, name, score, strand, effectSize, inputCount, outputCount, neglogPvalue, neglogQvalue, cohens_D]
+		peakToAdd = [chromo, start, end, name, score, strand, effectSize, inputCount, outputCount, neglogPvalue, neglogQvalue, cohens_D, peusdoLog2FC]
 
 		outputStream.write('\t'.join([str(x) for x in peakToAdd]) + "\n")
 	outputStream.close()
