@@ -6,6 +6,7 @@ import pyBigWig
 
 def setGlobalVariables(args):
 	setInputFiles(args.ctrlbw, args.expbw)
+	setNormalizedInputFiles(args.normCtrlbw, args.normExpbw)
 	setOutputDirectory(args.o)
 	setAnlaysisRegion(args.r, args.bl)
 	setFilterCriteria(args.fdr)
@@ -41,6 +42,26 @@ def setInputFiles(ctrlbwFiles, expbwFiles):
 
 	ALL_ZERO = [0] * SAMPLE_NUM
 
+def setNormalizedInputFiles(normCtrlbw, normExpbw):
+	global I_LOG2FC
+
+	if normCtrlbw is None or normExpbw is None:
+		I_LOG2FC = False
+		return
+	
+	I_LOG2FC = True
+	global NORM_CTRLBW_NAMES
+	global NORM_EXPBW_NAMES
+	
+	if  len(normCtrlbw) != CTRLBW_NUM or len(normExpbw) != EXPBW_NUM:
+		print("Error: The number of normalized observed bigwigs does not match with the number of input bigwigs. The number of bigwigs in -ctrlbw and -expbw should match with the nubmer of biwigs in -normCtrlbw and -normExpbw, respectively.")	
+		sys.exit()
+
+	NORM_CTRLBW_NAMES = normCtrlbw
+	NORM_EXPBW_NAMES = normExpbw
+
+
+
 
 def setOutputDirectory(outputDir):
 	global OUTPUT_DIR
